<?php

namespace DeliSkypress\WordPress\Services;

defined( 'ABSPATH' ) or die( 'Cheatin&#8217; uh?' );

use DeliSkypress\Models\TableInterface;
use DeliSkypress\Models\ServiceInterface;


class AbstractTable implements TableInterface, ServiceInterface{

    protected $charset = null;
    
    protected $alias = "";
    
    protected $tableName = "";

    protected $fields = array();

    /**
     * Quote a string that is used as an identifier
     * (table names, column names etc). This method can
     * also deal with dot-separated identifiers eg table.column
     */
    protected function quoteOneIdentifier($identifier) {
        $parts = explode('$-$', $identifier);
        $parts = array_map(array($this, 'quoteIdentifierPart'), $parts);
        return join('$-$', $parts);
    }

    /**
     * Quote a string that is used as an identifier
     * (table names, column names etc) or an array containing
     * multiple identifiers. This method can also deal with
     * dot-separated identifiers eg table.column
     */
    protected function quoteIdentifier($identifier) {
        if (is_array($identifier)) {
            $result = array_map(array($this, 'quoteOneIdentifier'), $identifier);
            return join(', ', $result);
        } else {
            return $this->quoteOneIdentifier($identifier);
        }
    }

    /**
     * This method performs the actual quoting of a single
     * part of an identifier, using the identifier quote
     * character specified in the config (or autodetected).
     */
    protected function quoteIdentifierPart($part) {
        if ($part === '*') {
            return $part;
        }

        $quote_character = apply_filters(DELIPRESS_SLUG . "_identifier_quote_character", "'");

        return $quote_character .
                str_replace($quote_character,
                            $quote_character . $quote_character,
                            $part
                ) . $quote_character;
    }

    /**
     * Override this function !
     * 
     * @param array $args
     * @return array
     */
    public function getFormatFields($args){
        return array();
    }

    /**
     * Prepare select SQL
     * @see TableInterface
     * @param array $args
     * @return string
     */
    public function getSelects($args){
        $sql = "";

        if($args["selects"] === "*"){
            $sql .= "{$this->alias}.*";
        }
        else if(is_array($args["selects"])){
            $sql .= "{$this->alias}." . implode(", {$this->alias}.", $args["selects"]);
        }

        return $sql;
    }

    /**
     * @see TableInterface
     */
    public function getFrom(){
        global $wpdb;
        
        return " FROM {$wpdb->prefix}{$this->tableName} {$this->alias} ";
    }


    /**
     * @see TableInterface
     */
    public function getQuery($queries){
        $sql = "";

        foreach($queries as $key => $query){
            if(!empty($query["value"])){
                switch($query["operator"]){
                    default:
                        if(!isset($query["type"])){
                            $query["type"] = "string";
                        }
                        switch($query["type"]){
                            case "string":
                            default:
                                $sql .= "AND {$this->alias}.{$query["field"]} = '{$query["value"]}' ";  
                                break;
                            case "datetime":
                                $sql .= "AND {$this->alias}.{$query["field"]} {$query["operator"]} '{$query["value"]}' ";  
                                break;
                            case "int":
                                $sql .= "AND {$this->alias}.{$query["field"]} {$query["operator"]} {$query["value"]} ";  
                                break;
                        }
                        break;
                    case "IN":
                        $valuesIn = $this->quoteIdentifier($query["value"]);
                        $sql .= "AND {$this->alias}.{$query["field"]} IN ({$valuesIn}) ";
                        break;
                }
            }
        }

        return $sql;
    }

    /**
     * @see TableInterface
     */
    public function createTable(){
        global $wpdb;

        if($this->charset === null){
            $this->charset = $wpdb->get_charset_collate();
        }
        if(empty($this->fields) || empty($this->tableName)){
            return;
        }

        $sql   = array();
        $sql[] = "CREATE TABLE {$wpdb->prefix}{$this->tableName} (";
        $sql   = array_merge($sql, $this->fields);
        $sql[] = ") {$this->charset};";
        

        if(!function_exists("dbDelta")){
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        }

        $sql = implode("\n", $sql);

        dbDelta( $sql );
    }


    /**
     * @see TableInterface
     */
    public function getTableName(){
        return $this->tableName;
    }

    /**
     * @see TableInterface
     */
    public function getFullTableName(){
        global $wpdb;

        return $wpdb->prefix . $this->tableName;
    }

    /**
     * @see TableInterface
     */
    public function getAlias(){
        return $this->alias;
    }

    /**
     * @see TableInterface
     */
    public function getOffsetLimitByPage($page, $limit = 20){
        if($page === null){
            return "";
        }

        $offset = ($page * $limit) - $limit;

        return "LIMIT {$limit} OFFSET {$offset}";
    }

    /**
     * @see TableInterface
     * @param array $fields
     * @param array $where
     * @return void
     */
    public function update($fields, $where){

        global $wpdb;

        $table        = $this->getFullTableName();
        $formatFields = $this->getFormatFields($fields);
        $whereFields  = $this->getFormatFields($where);
        
        $wpdb->update( 
            $table, 
            $fields, 
            $where, 
            $formatFields, 
            $whereFields
        );
    }

    public function delete($id){
        global $wpdb;

        $table        = $this->getFullTableName();

        $wpdb->delete( 
            $table, 
            array( 
                'id'    => $id,
            ), 
            array( 
                '%d'
            ) 
        );
    }

}
