<?php

namespace Delipress\WordPress\Models\AbstractModel;

defined( 'ABSPATH' ) or die( 'Cheatin&#8217; uh?' );

use Delipress\WordPress\Services\Table\TableServices;

use Delipress\WordPress\Models\InterfaceModel\ProviderImportInterface;
use Delipress\WordPress\Models\AbstractModel\AbstractProviderSettings;
use Delipress\WordPress\Models\ListModel;

use Delipress\WordPress\Traits\PrepareParams;

use Delipress\WordPress\Helpers\TaxonomyHelper;
use Delipress\WordPress\Helpers\ProviderHelper;
use Delipress\WordPress\Helpers\StatusHelper;

use Delipress\WordPress\Async\ImportBackgroundProcess;




/**
 * AbstractProviderImport
 *
 * @author Delipress
 * @version 1.0.0
 * @since 1.0.0
 */
abstract class AbstractProviderImport extends AbstractProviderSettings implements ProviderImportInterface {

    use PrepareParams;

    protected $fieldsPosts = array(
        "lists"         => "",
    );
    
    protected $fieldsRequired = array(
        "lists",
    );

    protected $providerApi = null;
    
    protected $range = 150;

    protected $callMetaData = false;

    /**
     * 
     * @param ImportBackgroundProcess $importBackgroundProcess
     * @return AbstractProviderImport
     */
    public function setBackgroundProcess($importBackgroundProcess){
        $this->importBackgroundProcess = $importBackgroundProcess;
        return $this;
    }

    /**
     *
     * @return int
     */
    public function getRange(){
        return apply_filters(DELIPRESS_SLUG . "_range_" . $this->provider["key"] . "_import", $this->range);
    }

    /**
     *
     * @param array $list
     * @return int
     */
    abstract protected function getIdFromListResult($list);

    /**
     *
     * @param array $list
     * @return string
     */
    abstract protected function getNameFromListResult($list);

    /**
     *
     * @param array $list
     * @return string
     */
    abstract protected function getTotalSubscriberFromListResult($list);

    /**
     *
     * @param array $subscriber
     * @return int
     */
    abstract protected function getIdSubscriberFromSubscribersResult($subscriber);

    /**
     *
     * @param array $subscriber
     * @return string
     */
    abstract protected function getEmailSubscriberFromSubscribersResult($subscriber);

    /**
     *
     * @param array $subscriber
     * @return string
     */
    abstract protected function getFirstNameSubscriberFromSubscribersResult($subscriber);

    /**
     *
     * @param array $subscriber
     * @return string
     */
    abstract protected function getLastNameSubscriberFromSubscribersResult($subscriber);

    /**
     *
     * @param array $subscriber
     * @return string
     */
    abstract protected function getCreatedAtSubscriberFromSubscribersResult($subscriber);

    protected function getProvider(){
        return $this->provider["key"];
    }
    

    protected function getSource(){
        return $this->sourceProvider;
    }
    

    protected function getSubscribers(ListModel $list, $offset, $limit){
        return $this->providerApi->getListContacts($list->getProviderId(), $offset, $limit); 
    }

    /**
     *
     * @param array $subscriber
     * @return bool
     */
    protected function getIsUnsubscribeFromSubscriberResult($subscriber){
        return false;
    }

    /**
     *
     * @param array $subscriber
     * @return bool
     */
    protected function getStatusSubscribeFromSubscriberResult($subscriber){
        return StatusHelper::SUBSCRIBE;
    }


    /**
     * @return array
     */
    public function importLists(){
        $provider = ProviderHelper::getProviderFromUrl();

        $response = $this->prepareListsForBackgroundProcess();

        if(!$response["success"] && !empty($response["results"])){
            $url = $this->optionServices->getPageUrl();
            wp_redirect($url);
            exit;
        }

        if(!isset($response["results"]) || empty($response["results"])){
            return;
        }

        foreach($response["results"] as $key => $list){
            $this->importBackgroundProcess->push_to_queue($list);
        }
        
		$this->importBackgroundProcess->save()->dispatch();

        return array(
            "success" => true
        );

    }

    /**
     *
     * @param ListModel $listModel
     * @return array
     */
    public function synchronizeListSubscriberForBackgroundProcess(ListModel $listModel){
        $provider = ProviderHelper::getProviderFromUrl();

        $list        = $this->providerApi->getList($listModel->getProviderId());
        
        if(!$list["success"]){
            return array(
                "success" => false
            );
        }
        $isLocal = delipress_is_local();
        
        if(!$isLocal){
            $this->providerWebhook->createWebhookOnList($listModel);
        }

        $range                 = $this->getRange();
        $totalSubscriberImport = $this->getTotalSubscriberFromListResult($list["results"]);

        $countCall    = ceil($totalSubscriberImport /  $range);
        $providerId   = $this->getIdFromListResult($list["results"]);

        for ($i=1; $i <= $countCall; $i++) { 
            $this->importBackgroundProcess->push_to_queue(
                array(
                    "provider"          => $this->provider["key"],
                    "offset"            => ($i*$range) - $range,
                    "limit"             => $range,
                    "list_id"           => $listModel->getId(),
                    "list_provider_id"  => $providerId,
                    "page"              => $i,
                    "total"             => $countCall
                )
            );
        }
        
        $this->importBackgroundProcess->save()->dispatch();

        return array(
            "success" => true
        );
    }


    /**
     * Prepare Import lists for background process
     * USE ONLY FOR IMPORT
     * 
     * @filter DELIPRESS_SLUG . "_range_" . $this->provider["key"] . "_import"
     * 
     * @return array
     */
    public function prepareListsForBackgroundProcess(){

        $params = $this->getPostParams("fields");

        if(!empty($this->missingParameters)){
            return array(
                "success" => false
            );
        }

        $prepareImportLists = array();
        $range              = $this->getRange();
        $providerMD5        = $this->optionServices->getMd5Provider();
        
        foreach($params["lists"] as $key => $listId){
            $list        = $this->providerApi->getList($listId);
            if($list["success"]){
                
                $providerId   = $this->getIdFromListResult($list["results"]);
                $name         = $newName = $this->getNameFromListResult($list["results"]);

                $termExist = get_term_by( "name", $name, TaxonomyHelper::TAXO_LIST, ARRAY_A );

                $i = 1;
                while($termExist){
                    $newName = $name . " ($i)";
                    $termExist = get_term_by( "name", $newName, TaxonomyHelper::TAXO_LIST, ARRAY_A );
                    $i++;
                }

                $term = wp_insert_term(
                    $newName,
                    TaxonomyHelper::TAXO_LIST
                );

                if(!is_wp_error( $term ) ){
                    update_term_meta(
                        $term["term_id"], 
                        TaxonomyHelper::META_LIST_ID_PROVIDER, 
                        sprintf("%s_%s", $this->provider["key"], $providerId) 
                    );

                    update_term_meta($term["term_id"], TaxonomyHelper::META_LIST_API_KEY_MD5, $providerMD5 );

                    $listModel = new ListModel();
                    $listModel->setListByTermId($term["term_id"]);

                    $isLocal = delipress_is_local();

                    if(!$isLocal){
                        $this->providerWebhook->createWebhookOnList($listModel);
                    }
                    
                    $totalSubscriberImport = $this->getTotalSubscriberFromListResult($list["results"]);

                    $countCall = ceil($totalSubscriberImport /  $range);
                    
                    for ($i=1; $i <= $countCall; $i++) { 
                        $prepareImportLists[] = array(
                            "provider"          => $this->provider["key"],
                            "offset"            => ($i*$range) - $range,
                            "limit"             => $range,
                            "list_id"           => $term["term_id"],
                            "list_provider_id"  => $providerId,
                            "page"              => $i,
                            "total"             => $countCall
                        );
                    }
                }


            }
        }

        return array(
            "success" => true,
            "results" => $prepareImportLists
        );
    }

    /**
     * 
     * @param ListModel $list
     * @param int $offset
     * @param int $limit
     * @return void
     */
    public function importListContacts(ListModel $list, $offset, $limit){
        $subscriberTableServices     = TableServices::getTable("SubscriberTableServices");
        $listSubscriberTableServices = TableServices::getTable("ListSubscriberTableServices");

        $subscribers                 = $this->getSubscribers($list, $offset, $limit);
        
        $prepareArgsInsert = array();
        $emails            = array();
        
        if(!$subscribers["success"]){
            return;
        }

        foreach($subscribers["results"] as $key => $subscriber){
            $emails[] = $this->getEmailSubscriberFromSubscribersResult($subscriber);
        }
        
        
        $emailSubscribersExist           = array();
        $args = array(
            "selects" => array(
                "id",
                "email",
                "provider_id"
            ),
            "query" => array(
                array(
                    "field"    => "email",
                    "operator" => "IN",
                    "value"    => $emails
                )
            )
        );
        $results                         = $subscriberTableServices->getSubscribers($args, null);
        foreach($results as $key => $result){
            $emailSubscribersExist[]     = $result["email"];
        }

        // Prepare subscribers not exist
        foreach($subscribers["results"] as $key => $subscriber){
            $email     = $this->getEmailSubscriberFromSubscribersResult($subscriber);
            $email     = sanitize_email( strtolower( $email) );
            
            if(!in_array($email, $emailSubscribersExist)){
                
                $createdAt = $this->getCreatedAtSubscriberFromSubscribersResult($subscriber);
                
                if($this->callMetaData){
                    $this->getSubscriberMetaData($subscriber);
                }
                
                $firstName = $this->getFirstNameSubscriberFromSubscribersResult($subscriber);
                $lastName  = $this->getLastNameSubscriberFromSubscribersResult($subscriber);
                $firstName = esc_sql($firstName);
                $lastName  = esc_sql($lastName);

                $prepareArgsInsert[] = array(
                    "first_name"       => $firstName,
                    "last_name"        => $lastName,
                    "email"            => $email,
                    "created_at"       => $createdAt->format("Y-m-d H:i:s"),
                    "provider_id"      => $this->getIdSubscriberFromSubscribersResult($subscriber),
                    "is_synchronize"   => 1
                );
                
            }
        }
        
        // Insert subscribers not exist
        if(!empty($prepareArgsInsert)){
            $subscriberTableServices->insertSubscribers($prepareArgsInsert);
        }

        $prepareArgsListSubscriber          = array();
        $prepareArgsListSubscriberUpdate    = array();
          
        foreach($subscribers["results"] as $key => $subscriber){
            $status    = $this->getStatusSubscribeFromSubscriberResult($subscriber);
            $email     = $this->getEmailSubscriberFromSubscribersResult($subscriber);
            $email     = sanitize_email( strtolower( $email) );

            $listSubscriber = $listSubscriberTableServices->getListSubscriberByListIdAndEmail($list->getId(), $email);
            if($status === StatusHelper::PENDING){
                continue;
            }

            if(empty($listSubscriber)){

                $subscriberResult = $subscriberTableServices->getSubscriberByEmail($email);

                if(empty($subscriberResult)){
                    continue;
                }
                
                $prepareArgsListSubscriber[] = array(
                    "term_taxonomy_id" => $list->getId(),
                    "subscriber_id"    => $subscriberResult["id"],
                    "provider"         => $this->getProvider(),
                    "provider_id"      => $this->getIdSubscriberFromSubscribersResult($subscriber),
                    "source"           => $this->getSource(),
                    "status"           => $status,
                    "is_synchronize"   => 1
                );
            }
            else{
                $listSubscriberTableServices->update(
                    array(
                        "provider"         => $this->getProvider(),
                        "provider_id"      => $this->getIdSubscriberFromSubscribersResult($subscriber),
                        "source"           => $this->getSource(),
                        "status"           => $status,
                        "is_synchronize"   => 1
                    ),
                    array(
                        "term_taxonomy_id" => $list->getId(),
                        "subscriber_id"    => $listSubscriber["subscriber_id"]
                    )
                );
            }
        }
        
        // Connect all list subscribers
        if(!empty($prepareArgsListSubscriber)){
            $listSubscriberTableServices->insertListSubscribers($prepareArgsListSubscriber);
        }
    }

}

