<?php

namespace Delipress\WordPress\Services\Listing;

defined( 'ABSPATH' ) or die( 'Cheatin&#8217; uh?' );

use DeliSkypress\Models\ServiceInterface;
use DeliSkypress\Models\MediatorServicesInterface;

use Delipress\WordPress\Models\ListModel;

use Delipress\WordPress\Helpers\TaxonomyHelper;
use Delipress\WordPress\Helpers\ProviderHelper;
use Delipress\WordPress\Helpers\ErrorFieldsNoticesHelper;
use Delipress\WordPress\Helpers\AdminNoticesHelper;
use Delipress\WordPress\Helpers\CodeErrorHelper;
use Delipress\WordPress\Helpers\AdminFormValues;

use Delipress\WordPress\Traits\PrepareParams;
use Delipress\WordPress\Traits\Listing\ListTrait;


/**
 * CreateListServices
 *
 * @author DeliPress
 * @version 1.0.0
 * @since 1.0.0
 */
class CreateListServices implements ServiceInterface, MediatorServicesInterface {

    use PrepareParams;
    use ListTrait;

    protected $paramsNotValid = 0;

    protected $missingParameters = array();

    protected $fieldsPosts = array(
        TaxonomyHelper::LIST_NAME         => "sanitize_text_field"
    );

    protected $fieldsPostMetas = array(
        TaxonomyHelper::META_LIST_COLOR   => "sanitize_text_field"
    );

    protected $fieldsRequired = array(
        TaxonomyHelper::LIST_NAME
    );

    protected $fieldsMetasRequired = array();

    /**
     * @see MediatorServicesInterface
     *
     * @param array $services
     * @return void
     */
    public function setServices($services){

        $this->optionServices          = $services["OptionServices"];
        $this->providerServices        = $services["ProviderServices"];
        $this->synchronizeListServices = $services["SynchronizeListServices"];

    }

    /**
     *
     * @param array $params
     * @param array $metas
     * @return void
     */
    protected function verifyParameters($params, $metas){

        if(empty($params[TaxonomyHelper::LIST_NAME])){
            $this->paramsNotValid++;
            ErrorFieldsNoticesHelper::registerError(
                TaxonomyHelper::LIST_NAME,
                CodeErrorHelper::getMessage(CodeErrorHelper::NOT_EMPTY)
            );
        }
    }

     /**
     * @action DELIPRESS_SLUG . "_before_create_list"
     * @action DELIPRESS_SLUG . "_after_create_list"
     *
     * @return array
     */
    public function createListStandalone($params, $metas){

        do_action(DELIPRESS_SLUG . "_before_create_list", $params, $metas);

        $term = wp_insert_term(
            $params[TaxonomyHelper::LIST_NAME],
            TaxonomyHelper::TAXO_LIST
        );

        if(is_wp_error( $term ) ){
            ErrorFieldsNoticesHelper::registerError(
                TaxonomyHelper::LIST_NAME,
                $term->get_error_message()
            );

            return array(
                "success" => false,
                "results" => array(
                    "message" => $term->get_error_message()
                )
            );
        }


        $list = new ListModel();
        $list->setListByTermId($term["term_id"]);

        $response = $this->synchronizeListServices
                            ->createListSynchronize($list);

        if(!$response["success"]){
            wp_delete_term( $term["term_id"], TaxonomyHelper::TAXO_LIST );
            return $response;
        }
        else{
            AdminNoticesHelper::registerSuccess(
                CodeErrorHelper::ADMIN_NOTICE,
                __("List successfully created", "delipress")
            );
        }
        
        add_term_meta($term["term_id"], TaxonomyHelper::META_LIST_COLOR, $metas[TaxonomyHelper::META_LIST_COLOR]);

        do_action(DELIPRESS_SLUG . "_after_create_list", $term, $params, $metas);

        return array(
            "success" => true
        );
    
    }

    /**
     * @return array
     */
    public function createList(){
        $params = $this->getPostParams("fields");
        $metas  = $this->getPostParams("meta");

        $this->verifyParameters($params, $metas);

        if(!empty($this->missingParameters)){
            AdminNoticesHelper::registerError(
                CodeErrorHelper::ADMIN_NOTICE,
                CodeErrorHelper::getMessage(CodeErrorHelper::TRY_CHEAT)
            );
        }
        if(
            $this->paramsNotValid > 0
        ){
            AdminNoticesHelper::registerError(
                CodeErrorHelper::ADMIN_NOTICE,
                CodeErrorHelper::getMessage(CodeErrorHelper::ADMIN_NOTICE_ERROR_DEFAULT)
            );
            return array(
                "success" => false
            );
        }

        $this->createListStandalone($params, $metas);
        
        AdminFormValues::cleanFormValues();

        return array(
            "success" => true
        );
    }

    /**
     *
     * @return array
     */
    public function editList(){

        $this->fieldsPosts = array(
            "list_id"                         => "checkListExist",
            TaxonomyHelper::LIST_NAME         => "sanitize_text_field",
        );

        $this->fieldsRequired = array(
            "list_id",
            TaxonomyHelper::LIST_NAME
        );

        $params = $this->getPostParams("fields");
        $metas  = $this->getPostParams("meta");

        if(!empty($this->missingParameters)){
            AdminNoticesHelper::registerError(
                CodeErrorHelper::ADMIN_NOTICE,
                CodeErrorHelper::getMessage(CodeErrorHelper::TRY_CHEAT)
            );
        }

        do_action(DELIPRESS_SLUG . "_before_delete_list", $params, $metas);

        $term = wp_update_term(
            $params["list_id"]->getId(),
            TaxonomyHelper::TAXO_LIST,
            array(
                "name" => $params[TaxonomyHelper::LIST_NAME]
            )
        );

        if(is_wp_error( $term ) ){
            AdminNoticesHelper::registerError(
                CodeErrorHelper::ADMIN_NOTICE,
                CodeErrorHelper::getMessage(CodeErrorHelper::ADMIN_NOTICE_ERROR_DEFAULT)
            );
            
            return array(
                "success" => false
            );
        }

        update_term_meta($params["list_id"]->getId(), TaxonomyHelper::META_LIST_COLOR, $metas[TaxonomyHelper::META_LIST_COLOR]);

        AdminNoticesHelper::registerSuccess(
            CodeErrorHelper::ADMIN_NOTICE,
            __("List successfully edited", "delipress")
        );

        do_action(DELIPRESS_SLUG . "_after_delete_list", $term, $params, $metas);

        AdminFormValues::cleanFormValues();

        return array(
            "success" => true
        );
    }
}
