<?php

namespace Delipress\WordPress\Services\Listing;

defined( 'ABSPATH' ) or die( 'Cheatin&#8217; uh?' );

use DeliSkypress\Models\ServiceInterface;
use DeliSkypress\Models\MediatorServicesInterface;

use Delipress\WordPress\Helpers\TaxonomyHelper;
use Delipress\WordPress\Helpers\ErrorFieldsNoticesHelper;
use Delipress\WordPress\Helpers\PageAdminHelper;
use Delipress\WordPress\Helpers\ActionHelper;
use Delipress\WordPress\Helpers\ProviderHelper;

use Delipress\WordPress\Models\ListModel;

/**
 * ListServices
 *
 * @author Delipress
 * @version 1.0.0
 * @since 1.0.0
 */
class ListServices implements ServiceInterface, MediatorServicesInterface {

    /**
     * @see MediatorServicesInterface
     * 
     * @param array $services
     * @return void
     */
    public function setServices($services){
        $this->providerServices = $services["ProviderServices"];
    }

    /**
     * 
     * @param array $args
     * @return Array ListModel
     */
    public function getLists($args = array()){
        global $wp_version;
        
        $lists = array();
        if(version_compare( $wp_version, '4.5.0', '>' ) ){
            $lists = get_terms( 
                array_merge(
                    array(
                        'taxonomy' => TaxonomyHelper::TAXO_LIST,
                        'hide_empty' => false,
                    ), $args
                )
            );
        }

        else{
            $lists = get_terms( TaxonomyHelper::TAXO_LIST, 
                array_merge(
                    array(
                        'hide_empty' => false,
                    ), $args
                )
            );
        }

        if($lists){
            foreach($lists as $key => $list){
                $value = new ListModel();
                $value->setList($list);

                $lists[$key] = $value;
            }
        }

        return $lists;
    }

    /**
     * 
     * @param array $args
     * @return int
     */
    public function getCountLists($args = array()){
        return wp_count_terms(TaxonomyHelper::TAXO_LIST,
            array_merge(
                array(
                    'hide_empty' => false,
                ), $args
            )
        );
    }

    /**
     *
     * @param string $provider
     * @param string $md5Provider
     * @param array $args
     * @return int
     */
    public function getCountListsOnlySynchronizeWithProvider($provider, $md5Provider, $args = array()){
        $this->getCountLists(
            array(
                "meta_query"  => array(
                    array(
                        'relation' => 'AND',
                        array(
                            'key'     => TaxonomyHelper::META_LIST_ID_PROVIDER,
                            'value'   => $provider,
                            'compare' => "LIKE",
                        ),
                        array(
                            'key'     => TaxonomyHelper::META_LIST_API_KEY_MD5,
                            'value'   => $md5Provider
                        )
                    )
                )
            )
        );
    }

    /**
     * Get lists already synchronize
     * 
     * @return array
     */
    public function getListsSynchronize(){
        return $this->getLists(
            array(
                "meta_query"  => array( 
                    array(
                        'key'     => TaxonomyHelper::META_LIST_ID_PROVIDER,
                        'compare' => "EXISTS",
                    )
                )
            )
        );
    }

    /**
     *
     * @param string $providerId
     * @return ListModel|null
     */
    public function getListByProviderId($provider, $providerId){
        $lists = $this->getLists(
            array(
                "meta_query"  => array( 
                    array(
                        'key'     => TaxonomyHelper::META_LIST_ID_PROVIDER,
                        'compare' => "=",
                        'value'   => $provider . "_" . $providerId
                    )
                )
            )
        );

        if(empty($lists)){
            return null;
        }

        return $lists[0];
    }

    /**
     * Filter results from getLists provider
     * 
     * @param string $provider
     * @param array $results
     * @return array
     */
    protected function filterListResultsProvider($provider, $results){

        $idKeyProvider   = "id";
        $idsListProvider = array();
        switch($provider){
            case ProviderHelper::MAILJET:
                $idKeyProvider = "ID";
                break;
        }

        if(empty($results)){
            return array();
        }

        foreach($results as $key => $list){
            $idsListProvider[] = $list[$idKeyProvider];
        }

        return $idsListProvider;

    }

    /**
     * @param string $provider
     * @return array
     */
    public function getListsOnlySynchronizeWithProvider($provider, $md5Provider, $args = array()){
        return $this->getLists(
            array_merge(
                array(
                    "meta_query"  => array(
                        "relation" => "OR",
                        array(
                            'relation' => 'AND',
                            array(
                                'key'     => TaxonomyHelper::META_LIST_ID_PROVIDER,
                                'value'   => $provider,
                                'compare' => "LIKE",
                            ),
                            array(
                                'key'     => TaxonomyHelper::META_LIST_API_KEY_MD5,
                                'value'   => $md5Provider
                            )
                        ),
                        array(
                            'key'     => TaxonomyHelper::META_LIST_API_KEY_MD5,
                            'compare' => "NOT EXISTS",
                        ),
                    )
                ),
                $args
            )
        );
    }

    /**
     * 
     * @param string $provider
     * @return array
     */
    public function getListsSynchronizeOrNotForProvider($provider, $params = array()){
        $lists =  $this->providerServices
                       ->getProviderApi($provider)
                       ->getLists($params);

        $idsListProvider = $this->filterListResultsProvider($provider, $lists["results"]);
        
        $listsWP = $this->getLists();
        $idsListsWPSynchronize = array();
        foreach($listsWP as $key => $list){
            $idsListsWPSynchronize[] = $list->getProviderId();
        }

        $listsToSynchronize = array_diff($idsListProvider, $idsListsWPSynchronize);
        $resultListsSynchronize = array();

        $idKeyProvider = "id";
        switch($provider){
            case ProviderHelper::MAILJET:
                $idKeyProvider = "ID";
                break;
            default:
                $idKeyProvider = apply_filters(DELIPRESS_SLUG . "_id_key_provider_" . $provider, $idKeyProvider);
                break;
        }
        
        foreach($lists["results"] as $key => $list){
            if(!in_array($list[$idKeyProvider], $listsToSynchronize)){
                $resultListsSynchronize[] = $list[$idKeyProvider];
            }
        }

        return array(
            "synchronize"    => $resultListsSynchronize,
            "lists"          => $lists["results"],
            "total_items"    => $lists["total_items"]
        );

    }


    /**
     * 
     * @param array $data
     */
    public function importLists($data){
        foreach ($data as $key => $term) {
            $response = wp_insert_term(
                $term->name,
                TaxonomyHelper::TAXO_LIST,
                array(
                    'description' => $term->description,
                    'slug'        => $term->slug,
                    'parent'      => $term->parent
                )
            );

            if(is_wp_error($response)){
                ErrorFieldsNoticesHelper::registerError($response->get_error_message(), "_import_list");
                wp_redirect(admin_url( 'admin.php?page=' . PageAdminHelper::PAGE_OPTIONS ) );
                exit;
                break;
            }
        }

        ErrorFieldsNoticesHelper::registerSuccess(__("List successfully imported", "delipress"), "_import_list");
        wp_redirect(admin_url( 'admin.php?page=' . PageAdminHelper::PAGE_OPTIONS ) );
        exit;
    }


    /**
     * 
     * @return string
     */
    public function getCreateUrl(){

        return add_query_arg(
            array(
                "page"   => PageAdminHelper::getPageNameByConst(PageAdminHelper::PAGE_LISTS),
                "action" => "create"
            ),
            admin_url("admin.php")
        );

    }

    /**
     * @param int $idList
     * @return string
     */
    public function getEditUrl($idList){

        return add_query_arg(
            array(
                "page"   => PageAdminHelper::getPageNameByConst(PageAdminHelper::PAGE_LISTS),
                "action" => "create",
                "list_id" => $idList
            ),
            admin_url("admin.php")
        );

    }

    /**
     * 
     * @return string
     */
    public function getPageListUrl(){

        return add_query_arg(
            array(
                "page"   => PageAdminHelper::getPageNameByConst(PageAdminHelper::PAGE_LISTS),
            ),
            admin_url("admin.php")
        );

    }

    /**
     * 
     * @param int $id
     * @return string
     */
    public function getSinglePageListUrl($id){
        return add_query_arg(
            array(
                "page"    => PageAdminHelper::getPageNameByConst(PageAdminHelper::PAGE_LISTS),
                "action"  => "subscribers",
                "list_id" => $id
            ),
            admin_url("admin.php")
        );
    }

    /**
     * 
     * @param int $listId
     * @return string
     */
    public function getDeleteListUrl($listId){
        return wp_nonce_url( 
            add_query_arg(
                array(
                    "action"      => ActionHelper::DELETE_LIST,
                    "list_id"     => $listId
                ),
                admin_url("admin-post.php")
            ),
            ActionHelper::DELETE_LIST
        );
    }

    /**
     * @return string
     */
    public function getDeleteListsUrl(){
        return wp_nonce_url( 
            add_query_arg(
                array(
                    "action"      => ActionHelper::DELETE_LISTS
                ),
                admin_url("admin-post.php")
            ),
            ActionHelper::DELETE_LISTS
        );
    }

    /**
     * @return string
     */
    public function getCreateListUrlFormAdminPost(){
        return wp_nonce_url( 
            add_query_arg(
                array(
                    "action"      => ActionHelper::CREATE_LIST
                ),
                admin_url("admin-post.php")
            ),
            ActionHelper::CREATE_LIST
        );
    }

    /**
     * @return string
     */
    public function getCreateSubscriberOnListUrlFormAdminPost(){
        return wp_nonce_url( 
            add_query_arg(
                array(
                    "action"      => ActionHelper::CREATE_SUBSCRIBER
                ),
                admin_url("admin-post.php")
            ),
            ActionHelper::CREATE_SUBSCRIBER
        );
    }

    /**
     *
     * @param ListModel $list
     * @return string
     */
    public function executeSynchronizeListSubscriberImport(ListModel $list){
        return wp_nonce_url( 
            add_query_arg(
                array(
                    "action"      => ActionHelper::SYNCHRONIZE_LIST_SUBSCRIBER_IMPORT,
                    "list_id"     => $list->getId()
                ),
                admin_url("admin-post.php")
            ),
            ActionHelper::SYNCHRONIZE_LIST_SUBSCRIBER_IMPORT
        );
    }   

    /**
     *
     * @param ListModel $list
     * @return string
     */
    public function executeSynchronizeListSubscriberExport(ListModel $list){
        return wp_nonce_url( 
            add_query_arg(
                array(
                    "action"      => ActionHelper::SYNCHRONIZE_LIST_SUBSCRIBER_EXPORT,
                    "list_id"     => $list->getId()
                ),
                admin_url("admin-post.php")
            ),
            ActionHelper::SYNCHRONIZE_LIST_SUBSCRIBER_EXPORT
        );
    }   
    
    /**
     * @return string
     */
    public function getImportSubscriberOnListUrlFormAdminPost($step = 1){
        switch($step){
            case 1:
                $action = ActionHelper::IMPORT_SUBSCRIBER_STEP_ONE;
                break;
            case 2:
                $action = ActionHelper::IMPORT_SUBSCRIBER_STEP_TWO;
                break;

        }

        return wp_nonce_url( 
            add_query_arg(
                array(
                    "action"      => $action
                ),
                admin_url("admin-post.php")
            ),
            $action
        );
    }


}









