<?php

namespace Delipress\WordPress\Services\Subscriber;

defined( 'ABSPATH' ) or die( 'Cheatin&#8217; uh?' );

use DeliSkypress\Models\ServiceInterface;
use DeliSkypress\Models\MediatorServicesInterface;

use Delipress\WordPress\Helpers\PostTypeHelper;
use Delipress\WordPress\Helpers\TaxonomyHelper;
use Delipress\WordPress\Helpers\ProviderHelper;
use Delipress\WordPress\Helpers\CodeErrorHelper;
use Delipress\WordPress\Helpers\SourceSubscriberHelper;
use Delipress\WordPress\Helpers\ErrorFieldsNoticesHelper;
use Delipress\WordPress\Helpers\AdminNoticesHelper;
use Delipress\WordPress\Helpers\StatusHelper;
use Delipress\WordPress\Helpers\AdminFormValues;

use Delipress\WordPress\Models\SubscriberModel;
use Delipress\WordPress\Models\OptinModel;
use Delipress\WordPress\Models\ListModel;

use Delipress\WordPress\Traits\PrepareParams;
use Delipress\WordPress\Traits\Listing\ListTrait;
use Delipress\WordPress\Traits\Subscriber\SubscriberTrait;

use Delipress\WordPress\Services\Table\TableServices;


/**
 * CreateSubscriberServices
 *
 * @author DeliPress
 * @version 1.0.0
 * @since 1.0.0
 */
class CreateSubscriberServices implements ServiceInterface, MediatorServicesInterface{

    use PrepareParams;
    use ListTrait;
    use SubscriberTrait;

    protected $missingParameters = array();

    protected $paramsNotValid = 0;

    protected $subscriberExist = null;

    protected $fieldsPosts = array(
        "list_id"          => "checkListExist",
        'subscriber_id'    => "checkSubscriberExist",
        "email"            => "checkSubscriberExistByEmail",
        "first_name"       => "sanitize_text_field",
        "last_name"        => "sanitize_text_field",
        "source"           => "sanitize_text_field"
    );
    
    protected $fieldsPostMetas = array();

    protected $fieldsRequired = array();

    protected $fieldsMetasRequired = array(
        "email",
        "source"
    );

    /**
     * @see MediatorServicesInterface
     * 
     * @param array $services
     * @return void
     */
    public function setServices($services){

        $this->optionServices                = $services["OptionServices"];
        $this->optinStatsServices            = $services["OptinStatsServices"];
        $this->providerServices              = $services["ProviderServices"];
        $this->subscriberServices            = $services["SubscriberServices"];
        $this->confirmSubscribeServices      = $services["ConfirmSubscribeServices"];
        $this->synchronizeSubscriberServices = $services["SynchronizeSubscriberServices"];

    }


    /**
     * Create Subscriber from a optin
     * 
     * @action DELIPRESS_SLUG . "_before_create_subscriber_on_list_from_optin"
     * @action DELIPRESS_SLUG . "_after_create_subscriber_on_list_from_optin"
     * 
     * 
     * @param OptinModel $optin
     * @param array $params
     * @return array
     */
    public function createSubscriberOnListsFromOptin(OptinModel $optin, $params){

        $this->checkSubscriberExistByEmail($params["email"]);

        $options         = $this->optionServices->getOptions();
        $provider        = $this->optionServices->getProvider();
        $lists           = $optin->getLists();
        $postId          = null;
        $response        = array(
            "success" => true   
        ); 

        do_action(DELIPRESS_SLUG . "_before_create_subscriber_on_list_from_optin", $optin, $params);

        $email = $params["email"];

        switch($provider["key"]){
            case ProviderHelper::MAILCHIMP:
                $status = StatusHelper::PENDING;
                break;
            default:
                $status = StatusHelper::UNSUBSCRIBE;
                break;
        }

        $subscriberIsConfirmed = false;
        
        // Create subscriber before if not exist
        if( $this->subscriberExist === null){
            
            $table = TableServices::getTable("SubscriberTableServices");

            $isConfirmed = 0;

            if(!$options["subscribers"]["double_optin"]){
                $isConfirmed = 1;
                $status      =  StatusHelper::SUBSCRIBE;
            }
            
            $postId = $table->insertSubscriber(
                array(
                    "email"        => $email,
                    "first_name"   => $params["first_name"],
                    "last_name"    => $params["last_name"],
                    "is_confirmed" => $isConfirmed
                )
            );


            if($options["subscribers"]["double_optin"]){
                switch($provider["key"]){
                    case ProviderHelper::MAILCHIMP:
                        foreach($lists as $key => $list){
                            $response = $this->synchronizeSubscriberServices->subscriberSynchronizeOnList($list, $postId, $email, 
                                array(
                                    "first_name"   => $params["first_name"],
                                    "last_name"    => $params["last_name"],
                                    "safeError" => true,
                                    "status"    => $status
                                )
                            );
                        }
                        break;
                    default:
                        $this->confirmSubscribeServices->sendConfirmSubscribe($postId, $email);
                        break;
                }
            }

        }
        // Update subscriber on list
        else{
            $postId                = $this->subscriberExist->getId();
            $subscriberIsConfirmed = $this->subscriberExist->getIsConfirmed();
            
            if($subscriberIsConfirmed){
                $status      = StatusHelper::SUBSCRIBE;
            }
        }


        $args  = array();
        $table = TableServices::getTable("ListSubscriberTableServices");
        foreach($lists as $key => $list){
            $result = $table->getSubscribers(
                array(
                    "query"          =>  array(
                        array(
                            "field"    => "term_taxonomy_id",
                            "operator" => "=",
                            "value"    => $list->getId(),
                            "type"     => "int"
                        ),
                        array(
                            "field"    => "subscriber_id",
                            "operator" => "=",
                            "value"    => $postId,
                            "type"     => "int"
                        )
                    ),
                    "links"            => array()
                )
            );

            if(empty($result)){
                $args[$key]["term_taxonomy_id"] = $list->getId();
                $args[$key]["subscriber_id"]    = $postId;
                $args[$key]["source"]           = $optin->getId();
                $args[$key]["status"]           = $status;

                if($response["success"]){
                    $args[$key]["is_synchronize"] = (isset($response["results"]["synchronize"])) ? $response["results"]["synchronize"] : 0;
                    $args[$key]["provider"]       = $provider["key"];
                    $args[$key]["provider_id"]    = (isset($response["results"]["provider_id"])) ? $response["results"]["provider_id"] : "";
                }
            }
        }
        
        if(!empty($args)){

            $table->insertListSubscribers($args);
        }

        // Stats conversion
        $this->optinStatsServices->incrementConvert($optin->getId());

        // Not double optin => Synchronize instant
        if(
            !$options["subscribers"]["double_optin"] ||
            $subscriberIsConfirmed
        ){
            foreach($lists as $key => $list){
                $this->synchronizeSubscriberServices->subscriberSynchronizeOnList($list, $postId, $email, 
                    array(
                        "safeError"    => true,
                        "first_name"   => $params["first_name"],
                        "last_name"    => $params["last_name"],
                    )
                );
            }
        }

        do_action(DELIPRESS_SLUG . "_after_create_subscriber_on_list_from_optin", $optin, $postId, $params);

        return $response;

    }

    /**
     * Create Subscriber
     * @example array(
     *      "subscriber" => SubscriberModel | null,
     *      "list"       => ListModel,
     *      "first_name" => "",
     *      "last_name"  => "",
     *      "email"      => "",
     *      "metas"      => array()
     * )
     * 
     * @param array $params
     * @return array
     */
    public function createSubscriber($params){
        $postId          = null;
        $response        = array(
            "success" => true   
        ); 


        // Create subscriber before if not exist
        if( !array_key_exists("subscriber", $params) || $params["subscriber"] == null || empty($params["subscriber"] ) ){
            
            $table = TableServices::getTable("SubscriberTableServices");

            $postId = $table->insertSubscriber(
                array(
                    'email'        => $params["email"],
                    "first_name"   => (isset($params["first_name"]) ) ? $params["first_name"] : "",
                    "last_name"    => (isset($params["last_name"]) ) ? $params["last_name"] : "",
                    "is_confirmed" => 1
                )
            );

            if(isset($params["metas"])){
                
                $tableMeta = TableServices::getTable("SubscriberMetaTableServices");
                
                $prepareArgs = array();
                foreach($params["metas"] as $key => $meta){
                    $prepareArgs[] = array(
                        "subscriber_id" => $postId,
                        "meta_id"       => $key,
                        "meta_value"    => $meta,
                    );
                }

                if(!empty($prepareArgs)){
                    $tableMeta->insertSubscriberMetas($prepareArgs);
                }
    
            }

        }
        // Update subscriber on list
        else{
            $postId = $params["subscriber"]->getId();
        }
        
        $table = TableServices::getTable("ListSubscriberTableServices");
        
        $response = $table->getListSubscriber(
            array(
                "query" => array(
                    array(
                        "value"    => $params["list"]->getId(),
                        "operator" => "=",
                        "field"    => "term_taxonomy_id",
                        "type"     => "int"
                    ),
                    array(
                        "value"    => $postId,
                        "operator" => "=",
                        "field"    => "subscriber_id",
                        "type"     => "int"
                    )
                )
            )
        );  

        AdminFormValues::cleanFormValues();

        if(!empty($response)){

            $table->update(
                array(
                    "status"           => StatusHelper::SUBSCRIBE,
                ),
                array(
                    "subscriber_id"    => $postId,
                    "term_taxonomy_id" => $params["list"]->getId()
                )
            );

            return array(
                "success" => true,
                "post_id" => $postId
            );
        }
        
        $table->insertListSubscriber(
            array(
                "subscriber_id"    => $postId,
                "term_taxonomy_id" => $params["list"]->getId(),
                "source"           => (isset($params["source"])) ? $params["source"] : SourceSubscriberHelper::ADMINISTRATION,
                "status"           => StatusHelper::SUBSCRIBE,
            )
        );
        
        

        return array(
            "success" => true,
            "post_id" => $postId
        );
    }

    /**
     *
     * @param int $listId
     * @param string $email
     * @param array $params
     * @return array
     */
    public function createSynchronizeSubscriber($listId, $email, $metas = array()){
        
        $params = array(
            "email"   => $email
        );
        
        $emailVerify = $this->checkSubscriberExistByEmail($email, "obj");
        if($emailVerify instanceof SubscriberModel){
            $params["subscriber"] = $emailVerify;
        }
    
        $list = $this->checkListExist($listId);
        if($list === null){
            return array(
                "success" => false,
                "results" => array(
                    "message" => __("List not exist", "delipress")
                )
            );
        }
    
        $params["list"] = $list;
        $params         = array_merge($metas, $params);
    
        $result = $this->createSubscriber($params);
        if(!$result["success"]){
            return $result;
        }
        
        if(!array_key_exists("post_id", $result)){
            return $result;
        }

        $response = $this->synchronizeSubscriberServices
                         ->subscriberSynchronizeOnList($list, $result["post_id"], $email, $params);
        
        return $response;
    }


    /**
     * Create Subscriber from administration
     * @example array(
     *      "subscriber" => SubscriberModel | null,
     *      "list_id"    => ListModel,
     *      "first_name" => "",
     *      "last_name"  => "",
     *      "email"      => "",
     *      "metas"      => array()
     * )
     * @param array $params
     * @return array
     */
    public function createSubscriberOnListFromAdministration($params){
        
        $prepareParams         = $params;
        $prepareParams["list"] = $params["list_id"];

        if( $this->subscriberExist != null){
            $prepareParams["subscriber"] = $this->subscriberExist;
        }

        $result   = $this->createSubscriber($prepareParams);

        if(!array_key_exists("post_id", $result)){
            return $result;
        }

        $response = $this->synchronizeSubscriberServices->subscriberSynchronizeOnList($params["list"], $result["post_id"], $params["email"], $params);

        return $response;
    }
    
    /**
     * @action DELIPRESS_SLUG . "_before_edit_subscriber_on_list"
     * @action DELIPRESS_SLUG . "_after_edit_subscriber_on_list"
     * 
     * @return void
     */
    public function editSusbcriberOnList(){

        $this->fieldsRequired = array(
            "subscriber_id",
            "email"
        );


        $params = $this->getPostParams("fields");

        do_action(DELIPRESS_SLUG . "_before_edit_subscriber_on_list", $params);
        
        $this->verifyParameters($params);

        if(
            !empty($this->missingParameters) ||
            $this->paramsNotValid > 0
        
        ){
            return array(
                "success" => false
            );
        }

        $response = $this->editSubscriber($params);

        do_action(DELIPRESS_SLUG . "_after_edit_subscriber_on_list", $response);

        return $response;

    }

    /**
     *
     * @param int $listId
     * @param int $subscriberId
     * @param array $params
     * @param array $metas
     * @return array
     */
    public function editSynchronizeSubscriber($listId, $subscriberId, $params, $metas = array()){
        
        $prepareParams = array();
        $subscriber    = $this->checkSubscriberExist($subscriberId);
        if($subscriber instanceof SubscriberModel){
            $prepareParams["subscriber_id"] = $subscriber;
        }
        else{
            return array(
                "success" => false,
                "results" => array(
                    "message" => __("Subscriber not exist", "delipress")
                )
            );
        }
    
    
        $list = $this->checkListExist($listId);
        if($list === null){
            return array(
                "success" => false,
                "results" => array(
                    "message" => __("List not exist", "delipress")
                )
            );
        }
    
        if(empty($params)){
            return array(
                "success" => true,
                "results" => array(
                    "message" => __("No change", "delipress")
                )
            );
        }
    
        $prepareParams["list_id"] = $list;
        $prepareParams["metas"]   = $metas;
    
        $filter = array_filter(array_keys($params), function($k) {
            return in_array($k, array("first_name","last_name","email"));
        }); // Compatibility PHP 5.4
    
        $fields        = array_intersect_key($params, array_flip($filter));
        $prepareParams = array_merge($fields, $prepareParams);

        return $this->editSubscriber($prepareParams, false);
    }

    /**
     *
     * @param array $params
     * @example array(
     *      "subscriber_id" => SubscriberModel,
     *      "list_id" => ListModel,
     *      "metas" => array(),
     *      "email" => "johndoe@gmail.com",
     *      "first_name" => "John",
     *      "last_name"  => "Doe" 
     * )
     * @return array
     */ 
    public function editSubscriber($params, $withNotice = true){
        
        $table = TableServices::getTable("SubscriberTableServices");
        
        if(isset($params["email"])){
            $subscriberExist = $table->getSubscriberByEmail($params["email"]);
    
            if($subscriberExist && $params["subscriber_id"]->getId() != $subscriberExist["id"]){
                if(!$withNotice){
                    return array(
                        "success" => false,
                    );
                }

                AdminNoticesHelper::registerError(
                    CodeErrorHelper::ADMIN_NOTICE,
                    sprintf(__("A subscriber with email : %s already exists", "delipress"), $params["email"])
                );
    
                return array(
                    "success" => false,
                );
            }
        }

        $where = array(
            "id" => $params["subscriber_id"]->getId()
        );

        $filter = array_filter(array_keys($params), function($k) {
            return in_array($k, array("first_name","last_name","email"));
        }); // Compatibility PHP 5.4

        $fields = array_intersect_key($params, array_flip($filter));

        $table->update($fields, $where);

        $result = $this->synchronizeSubscriberServices->editSubscriberSynchronizeOnList($params["list_id"], $params["subscriber_id"], $fields);

        if($result["success"]){
            if($withNotice){
                AdminNoticesHelper::registerSuccess(
                    CodeErrorHelper::ADMIN_NOTICE,
                    __("Subscriber updated", "delipress") 
                );
            }
        }
        else{
            $provider        = $this->optionServices->getProvider();
            if($withNotice){
                AdminNoticesHelper::registerSuccess(
                    CodeErrorHelper::ADMIN_NOTICE,
                    sprintf(__("An error occurred during the update on %s ", "delipress"), $provider["key"] ) 
                );
            }
        }

        AdminFormValues::cleanFormValues();

        return $result;
    }


    /**
     * 
     * @param args $params
     * @return void
     */
    protected function verifyParameters($params){

        if(empty($params["email"])){
            $this->paramsNotValid++;
            ErrorFieldsNoticesHelper::registerError(
                CodeErrorHelper::META_SUBSCRIBER_EMAIL,
                __("Email field can not be empty", "delipress")
            );
        }
    }

    /**
     * @action DELIPRESS_SLUG . "_before_create_subscriber_on_list"
     * @action DELIPRESS_SLUG . "_after_create_subscriber_on_list"
     * 
     * @return void
     */
    public function createSubscriberOnList(){
        $this->fieldsRequired = array(
            "list_id",
            "email"
        );

        $params = $this->getPostParams("fields");

        do_action(DELIPRESS_SLUG . "_before_create_subscriber_on_list", $params);
        
        $this->verifyParameters($params);

        if(
            !empty($this->missingParameters) ||
            $this->paramsNotValid > 0
        
        ){
            return array(
                "success" => false
            );
        }

        $response = $this->createSubscriberOnListFromAdministration($params);
        

        $result = array(
            "success" => true,
            "results" => array(
                "list"        => $params["list_id"],
            )
        );

        if($response["success"]){
            AdminNoticesHelper::registerSuccess(
                CodeErrorHelper::ADMIN_NOTICE,
                CodeErrorHelper::getMessage(CodeErrorHelper::CREATE_SUBSCRIBER_ON_LIST_SUCCESS)
            );
        }


        do_action(DELIPRESS_SLUG . "_after_create_subscriber_on_list", $result);

        return $result;

    }
}

