<?php

namespace Delipress\WordPress\Services\Subscriber;

defined( 'ABSPATH' ) or die( 'Cheatin&#8217; uh?' );

use DeliSkypress\Models\ServiceInterface;
use DeliSkypress\Models\MediatorServicesInterface;

use Delipress\WordPress\Helpers\PostTypeHelper;
use Delipress\WordPress\Helpers\TaxonomyHelper;
use Delipress\WordPress\Helpers\ProviderHelper;
use Delipress\WordPress\Helpers\SourceSubscriberHelper;
use Delipress\WordPress\Helpers\DataEncoder;
use Delipress\WordPress\Helpers\ActionHelper;

use Delipress\WordPress\Models\SubscriberModel;
use Delipress\WordPress\Models\ListModel;

use Delipress\WordPress\Traits\PrepareParams;
use Delipress\WordPress\Traits\Listing\ListTrait;
use Delipress\WordPress\Traits\Subscriber\SubscriberTrait;

/**
 * UnsubscribeServices
 *
 * @author DeliPress
 * @version 1.0.0
 * @since 1.0.0
 */
class UnsubscribeServices implements ServiceInterface, MediatorServicesInterface{

    use ListTrait;
    use SubscriberTrait;
    use PrepareParams;

    protected $missingParameters = array();

    protected $paramsNotValid = 0;

    protected $subscriberExist = null;

    /**
     * @see MediatorServicesInterface
     * 
     * @param array $services
     * @return void
     */
    public function setServices($services){

        if(!array_key_exists("ProviderServices", $services)){
            throw new Exception("Miss ProviderServices");
        }

        $this->optionServices             = $services["OptionServices"];
        $this->providerServices           = $services["ProviderServices"];
        $this->emailHtmlServices          = $services["EmailHtmlServices"];
        $this->subscriberTableServices    = $services["SubscriberTableServices"];

    }

    /**
     * 
     * @param string $provider
     * @return string
     */
    public function getCodeUnsubscribeProvider($provider){
        $fallback = null;
        switch($provider){
            case ProviderHelper::MAILCHIMP:
                $fallback = "*|UNSUB|*";
                break;
            case ProviderHelper::MAILJET:
                $fallback = "[[UNSUB_LINK]]";
                break;
        }

        return $fallback;
    }

    /**
     * 
     * @param int $list
     * @return string
     */
    public function getRequestUnsubscribeFormUrl($listId){
        return wp_nonce_url(
            add_query_arg( 
                array(
                    "action" => ActionHelper::REQUEST_UNSUBSCRIBE,
                    "list_id" => $listId
                ),
                admin_url( 'admin-ajax.php' )
            ),
            ActionHelper::REQUEST_UNSUBSCRIBE
        );
    }

    /**
     * 
     * @param ListModel $list
     * @param SubscriberModel $subscriber
     * @param string $provider
     * @return string
     */
    public function getUnsubscribeUrl(ListModel $list, SubscriberModel $subscriber, $provider){

        $fallback = $this->getCodeUnsubscribeProvider($provider);

        return wp_nonce_url( 
            add_query_arg(
                array(
                    "data"   => DataEncoder::encodeData(
                        array(
                            "list_id"          => $list->getId(),
                            "subscriber_id"    => $subscriber->getId(),
                            "subscriber_email" => $subscriber->getEmail(),
                        )
                    ),
                    "fallback" => $fallback,
                    "token"    => $subscriber->getTokenUnsubscribe(false),
                    "action"   => ActionHelper::UNSUBSCRIBE
                ),
                home_url()
            ), 
            ActionHelper::UNSUBSCRIBE
        );
    }


    /**
     * Generate token for unsubscribe
     * 
     * @param ListModel $list
     * @param SubscriberModel $subscriber
     * @return string
     */
    protected function generateToken(ListModel $list, SubscriberModel $subscriber){
        return $md5 = md5( $subscriber->getId() . $list->getId() . $subscriber->getEmail() . $subscriber->getSalt() );
    }

    /**
     * 
     * @param string $token
     * @param ListModel $list
     * @param SubscriberModel $subscriber
     * @return bool
     */
    public function verifyTokenUnsubscribe($token, ListModel $list, SubscriberModel $subscriber){
        $md5 = $this->generateToken($list, $subscriber);

        return ($token === $md5);
    }

    /**
     * 
     * @param ListModel $list
     * @param SubscriberModel $subscriber
     * @return void
     */
    public function generateTokenUnsubscribe(ListModel $list, SubscriberModel $subscriber){
        $md5 = $this->generateToken($list, $subscriber);

        $this->subscriberTableServices->update(
            array(
                "token_unsubscribe" => $md5
            ),
            array(
                "id" => $subscriber->getId()
            )
        );
    }

    /**
     * Send email for unsubscribe on list
     * 
     * @return array
     */
    public function sendRequestUnsubscribe(){

        $this->fieldsPosts = array(
            "email"   => "checkSubscriberExistByEmail",
            "list_id" => "checkListExist"
        );

        $this->fieldsRequired = array(
            "email",
            "list_id"
        );

        $params = $this->getPostParams("fields");

        do_action(DELIPRESS_SLUG . "_before_send_request_unsubscribe", $params);

        if(
            !empty($this->missingParameters)
        ){
            return array(
                "success" => false
            );
        } 

        $provider    = $this->optionServices->getProvider();

        if(!$provider["is_connect"]){
            return array(
                "success" => false
            );

        }
       

        $providerApi = $this->providerServices->getProviderApi($provider["key"]);

        ob_start();
        include_once DELIPRESS_PLUGIN_DIR_EMAILS . "/unsubscribe.php";
        $html = ob_get_contents(); 
        ob_end_clean();

        $html = $this->emailHtmlServices->prepareEmailHtml($html);

        if(!$this->subscriberExist){
            $sendEmail = $params["email"];
            $fallback = $this->getCodeUnsubscribeProvider($provider);
            $html = str_replace("[delipress_request_unsubscribe]", $fallback, $html);
            
        }
        else{
            $sendEmail = $this->subscriberExist->getEmail();
            $this->generateTokenUnsubscribe($params["list_id"], $this->subscriberExist);
            $html = str_replace("[delipress_request_unsubscribe]", $this->getUnsubscribeUrl($params["list_id"], $this->subscriberExist, $provider["key"]), $html);
        }

        $response = $providerApi->sendEmail(
            array(
                "subject" => __("Unsubscribe", "delipress"),
                "html"    => $html,
                "emails" => array(
                    $sendEmail
                )
            )
        );

        do_action(DELIPRESS_SLUG . "_after_send_request_unsubscribe", $params);
        
        return $response;

    }


    /**
     * 
     * @param int $listId
     * @return string
     */
    public function getRequestUnsubscribeUrl($listId, $provider){

        return add_query_arg( 
            array(
                "action"   => ActionHelper::REQUEST_UNSUBSCRIBE,
                "list_id"  => $listId
            ),
            home_url()
        );
    }

}









