<?php

namespace Delipress\WordPress\Services\Table;

defined( 'ABSPATH' ) or die( 'Cheatin&#8217; uh?' );

use DeliSkypress\WordPress\Services\AbstractTable;
use DeliSkypress\Models\MediatorServicesInterface;

use Delipress\WordPress\Helpers\TableHelper;
use Delipress\WordPress\Helpers\ProviderHelper;

/**
 * MetaTableServices
 *
 * @author Delipress
 * @version 1.0.0
 * @since 1.0.0
 */
class MetaTableServices extends AbstractTable implements MediatorServicesInterface {

    protected $charset = null;

    protected $tableName = TableHelper::TABLE_META;

    protected $alias = "sm";

    protected $fields = array(
        "id bigint(20) unsigned NOT NULL AUTO_INCREMENT,",
        "name varchar(80),",
        "provider varchar(255) DEFAULT NULL,",
        "provider_id longtext,",
        "created_at datetime NOT NULL DEFAULT '0000-00-00 00:00:00',",
        "PRIMARY KEY (id)"
    );

    public function setServices($services){
        $this->optionServices   = $services["OptionServices"];
        $this->providerServices = $services["ProviderServices"];
    }

    /**
     * Get args default
     * 
     * @return array
     */
    protected function getArgsInsertDefault(){
        $now         = new \DateTime("now");

        return array(
            "name"        => "",
            "provider"    => "",
            "provider_id" => "",
            "created_at"  => $now->format("Y-m-d H:i:s")
            
        );
    }

    /**
     * Prepare values insert SQL
     * 
     * @param array $args
     * @return string
     */
    protected function insertMetaValues($args){

        $argsDefault = $this->getArgsInsertDefault();

        $args = array_merge($argsDefault, $args);

        return "(
            '{$args["name"]}',
            '{$args["provider"]}',
            '{$args["provider_id"]}',
            '{$args["created_at"]}'
        )";
    }

    /**
     * Prepare begin insert SQL
     * @return string
     */
    protected function insertMetaInstruction(){
        global $wpdb;

        return "
            INSERT INTO {$wpdb->prefix}{$this->tableName} 
            (
                name,
                provider,
                provider_id,
                created_at
            ) VALUES 
        ";
    }

    /**
     * @see AbstractTable
     * 
     * @param array $args
     * @return array
     */
    public function getFormatFields($args){
        $formats = array();
        foreach($args as $key => $arg){
            switch($key){
                case "name":
                case "created_at":
                case "provider":
                case "provider_id":
                    $formats[] = "%s";
                    break;
            }
        }
        
        return $formats;
    }

    /**
     * @action DELIPRESS_SLUG . "_before_insert_meta"
     * @action DELIPRESS_SLUG . "_after_insert_meta"
     * 
     * @param array $args
     * @return void
     */
    public function insertMeta($args = array()){
        
        do_action(DELIPRESS_SLUG . "_before_insert_meta", $args);
        
        global $wpdb;
        
        $table       = $wpdb->prefix . $this->tableName;
        
        $argsDefault = $this->getArgsInsertDefault();
        $args        = array_merge($argsDefault, $args);
        
        $response = $this->insertMetaToProvider($args["name"], $args);
        if($response["success"]){
            $args["provider_id"] = $response["results"]["provider_id"];
            $args["provider"]    = $response["results"]["provider"];
        }
        
        $formats = $this->getFormatFields($args);

        $wpdb->insert( 
            $table,
            $args, 
            $formats
        );

        do_action(DELIPRESS_SLUG . "_after_insert_meta", $args, $wpdb->insert_id);

        return $wpdb->insert_id;

    }

    /**
     *
     * @param string $name
     * @param array $params
     * @return array
     */
    public function insertMetaToProvider($name, $params){
        $provider         = $this->optionServices->getProvider();
        
        
        $synchronize = false;
        if(!$provider["is_connect"]){
            return array(
                "success" => false
            );
        }

        switch($provider["key"]){
            case ProviderHelper::MAILJET:
                $synchronize = true;
                break;
            default:
                $synchronize = apply_filters(DELIPRESS_SLUG . "_insert_meta_to_provider_" . $provider["key"], false);
                break;
        }

        if(!$synchronize){
            return array(
                "success" => false
            );
        }
        
        $providerApi = $this->providerServices
                            ->getProviderApi($provider["key"])
                            ->setSafeError(true);

        $params = array(
            "datatype" => (isset($params["datatype"])) ? $params["datatype"] : "str",
            "name"     => $name,
            "type"     => (isset($params["type"])) ? $params["type"] : "static",
        );

        $response = $providerApi->createMetaData($params);

        if(!$response["success"]){
            $response = $providerApi->getMetaDataByName($name);
        }

        if(!$response["success"]){
            return $response;
        }

        $result = array(
            "success" => true
        );

        switch($provider["key"]){
            case ProviderHelper::MAILJET:
                $result["results"] = array(
                    "provider_id" => $response["results"][0]["ID"],
                    "datatype"    => $response["results"][0]["Datatype"],
                    "type"        => $response["results"][0]["NameSpace"],
                    "provider"    => ProviderHelper::MAILJET
                );
                break;
            default:
                $result["results"] = apply_filters(DELIPRESS_SLUG . "_results_create_meta_data_" . $provider["key"], $response);
                break;
        }
        
        return $result;
    }

    /**
     * 
     * @action DELIPRESS_SLUG . "_before_insert_metas"
     * @action DELIPRESS_SLUG . "_after_insert_metas"
     * 
     * @param array $metas
     * @return void
     */
    public function insertMetas($metas){

        do_action(DELIPRESS_SLUG . "_before_insert_metas", $metas);

        global $wpdb;

        $sql = $this->insertMetaInstruction();
        
        $i = 1;
        $total = count($metas);
        foreach($metas as $key => $meta){
            
            $response = $this->insertMetaToProvider($meta["name"], $metas);
            if($response["success"]){
                $meta["provider_id"] = $response["results"]["provider_id"];
                $meta["provider"]    = $response["results"]["provider"];
            }

            $sql .= $this->insertMetaValues($meta);

            if($i < $total){
                $sql .= ",";
            }
            $i++;
        }

        $wpdb->query($sql);

        do_action(DELIPRESS_SLUG . "_after_insert_metas", $metas);
    }   


    /**
     * 
     * @param int $id
     * @return void
     */
    public function deleteMeta($id){
        $this->delete($id);
    }

  
    /**
     * 
     * @param array $args
     * @param int $page
     * @param int $limit
     * @return string
     */
    protected function getSqlMetas($args = array(), $page = 1, $limit = 20){

        global $wpdb;

        $argsDefault = array(
            "selects"      => "*",
            "query"        => array(
                array(
                    "field"    => "id",
                    "operator" => "=",
                    "value"    => ""
                )
            ),
            "limit"        => 20,
        );

        $args = array_merge($argsDefault, $args);

        $sql = "SELECT ";

        $sql .= $this->getSelects($args);

        $sql .= $this->getFrom();
        $sql .= "WHERE 1=1 ";

        $sql .= $this->getQuery($args["query"]);

        $sql .= $this->getOffsetLimitByPage($page, $limit);


        return $sql;
    }

    /**
     * 
     * @param array $args
     * @param int $page
     * @param int $limit
     * @return array
     */
    public function getMetas($args = array(), $page = 1, $limit = 20){
        global $wpdb;

        $sql = $this->getSqlMetas($args, $page, $limit);

        $metas = $wpdb->get_results($sql, ARRAY_A);

        return $metas;

    }


    /**
     * 
     * @param array $args
     * @return array
     */
    public function getMeta($args){
        global $wpdb;

        $sql = $this->getSqlMetas($args, null, null);

        return $wpdb->get_row($sql, ARRAY_A);

    }

    /**
     * 
     * @param int $id
     * @return array
     */
    public function getMetaById($id){
        global $wpdb;

        $sql = $this->getSqlMetas(
            array(
                "query" => array(
                    array(
                        "field"    => "id",
                        "operator" => "=",
                        "value"    => $id,
                        "type"     => "int"
                    )
                )
            ), 
            null
        );

        $meta = $wpdb->get_row($sql, ARRAY_A);

        return $meta;

    }

    /**
     * 
     * @param string $name
     * @return array
     */
    public function getMetaByName($name){
        global $wpdb;

        $sql = $this->getSqlMetas(
            array(
                "query" => array(
                    array(
                        "field"    => "name",
                        "operator" => "=",
                        "value"    => $name,
                    )
                )
            ), 
            null
        );

        $meta = $wpdb->get_row($sql, ARRAY_A);

        return $meta;

    }

}









